
% This is the main Matlab program for running imaging polarimetric self-calibration 
% on the "Corridor" data sequence. It can serve as a template for running the algorithm
% on other, simlar data, with code adaptations. The matlab code is available for 
% non-commercial use and exploration. 
% You are very welcome to use this code. For this, clearly acknowledge 
% the source of this code, and cite the paper that describes the algorithm:
% Y. Y. Schechner, "Self-Calibrating Imaging Polarimetry", 
% Proc. IEEE Int. Conference on Computational Photography (ICCP), 2015.
%
% (c) Copyright Yoav Schechner 2015. For commercial use contact the
% author. The author is not liable for any damages or loss that might be 
% caused by use or connection to this code. 


close all
clear

% Tolerance of error (in degrees) around the correct angles, from which initial guesses
% of these angles are randomly picked.
%
angle_rough_error=60;

% Geometric data about rectangular regions pre-marked by the mouse,
% The file names and the true polarizer angles in all files.
%
load rectangles_corridor_all

% Reasonable regions, that don't have saturation or hard undervalues
%
acceptable_regions=[1,2,4,5,7,8,9,10,11,12,13,15,16,17,18,19,20,24,26,29,30];

% Reading the radiance in the rectangles, into a data forma, using a
% function
%
[Idata,cangles]=read_selected_regions_lab(fprefix,cangles,rect_coord,acceptable_regions);

%  The corridor experiment included a lot of sampling angles. Here, we
%  select angles in a single semi-circle
%
subangle_inds=[10 12 14 16 17]; 
selected_angles=mod(cangles(subangle_inds),180);

true_angles=selected_angles
subdata=Idata(subangle_inds,:);

num_of_angles=length(selected_angles);
Idata_cs=subdata(:);
num_of_pixels=size(Idata,2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initializing the guessed polarizer angles, around the true angles.
% the first angle is set to zero anyway: only the others are estimated
% afterwards

initial_error= round(angle_rough_error*(rand(size(true_angles))-0.5));
initial_angles= true_angles+initial_error;
initial_angles(1)=0

[true_Object_matrix,true_object_angle_degrees,true_p_object,baseE]   = object_polariz3(selected_angles,num_of_pixels,Idata_cs,subdata);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First object estimation, using the initial random angles

yit=1;
[Object_matrix,recovered_object_angle_degrees,recovered_p_object,Emat(yit)]=object_polariz3(initial_angles,num_of_pixels,Idata_cs,subdata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A full iteration

[estimated_analyzer_angles_1iteration, estimated_polarizance ,hatP,MSEmat(yit)]=optimizeP_givenO(subdata,Object_matrix);
yit=yit+1;
[Object_matrix,recovered_object_angle_degrees,recovered_p_object,Emat(yit)]=object_polariz3(estimated_analyzer_angles_1iteration,num_of_pixels,Idata_cs,subdata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Many repeated iterations

maxiter=100;
for yiter=1:(maxiter-2);
     [estimated_analyzer_angles, estimated_polarizance ,hatP,MSEmat(yit)]=optimizeP_givenO(subdata,Object_matrix);
     estimated_analyzer_angles;
     yit=yit+1;
     [Object_matrix,recovered_object_angle_degrees,recovered_p_object,Emat(yit)]=object_polariz3(estimated_analyzer_angles,num_of_pixels,Idata_cs,subdata);
end
estimated_analyzer_angles_100iterations=round(10*estimated_analyzer_angles)/10


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Display of results

figure(9)
  hold off
  plot(true_object_angle_degrees,recovered_object_angle_degrees,'o','Linewidth',2)
  hold on
  plot(0:180,0:180,'Linewidth',1);
  axis image


figure(10)
  hold off
  plot(true_p_object,recovered_p_object,'+','Linewidth',2)
  hold on
  plot(0:0.1:1,0:0.1:1,'Linewidth',1);
  axis image

% 
% figure(13)
%  semilogx(MSEmat/baseE,'Linewidth',2)
%  set(gca,'xlim',[1,maxiter])
 
figure(14)
 loglog(Emat/baseE,'Linewidth',2)
 set(gca,'xlim',[1,maxiter])

 
 
